# -*- coding: utf-8 -*-
# Runge-Kutta 4 Verfahren für Räuber-Beute-Modell
# d/dt x = a*x-b*x*y
# d/dt y= -d*y+c*x*y
import numpy as np
import matplotlib.pyplot as plt

a = 2.0 #Reproduktionsrate der Beutepopulation x (ohne Räuber mit unendlichen Nahrungsresourcen)
d = 0.8 #
b = 0.02 #Sterberate der Beute pro Räuber ~ Fressrate der Räuber pro Beute
c = 0.0002 #Reproduktionsrate der Räuber pro Beute
p = np.array([5000.0,120.0]) #Anfangsbedingungen, 5000 Beutetiere, 120 Räuber

t = 0.0
h = 0.01
N_t = 20000
T_End = N_t*h
epsilon=0.4

def rhs(p,a,d,b,c,t):
  result = np.array([0.0,0.0])
  result[0] = p[0]*(a*(1.0+epsilon*np.sin(np.pi*t))-b*p[1])
  result[1] = -p[1]*(d-c*p[0])
  return result


resultarray = np.zeros((3,N_t)) # components are time, population1 and population2

for i in range(N_t):
  k1 = rhs(p,a,d,b,c,t)
  k2 = rhs(p+h/2.0*k1,a,d,b,c,t+0.5*h)
  k3 = rhs(p+h/2.0*k2,a,d,b,c,t+0.5*h)
  k4 = rhs(p+h*k3,a,d,b,c,t+h)
  p = p+h/6.0*(k1 + 2.0*k2 + 2.0*k3 + k4)
  resultarray[0,i] = t
  resultarray[1,i] = p[0]
  resultarray[2,i] = p[1]
  t = t+h

# search for all maxima of y(t):
maxr = []
maxt = []
for j in range(100, len(resultarray[1,:])-1):
	if resultarray[1,j] > resultarray[1,j-1] and resultarray[1,j] > resultarray[1,j+1]:
		maxr.append(resultarray[1,j])
		maxt.append(resultarray[0,j])

fig = plt.figure(1)
plt.xlabel("t",fontsize=16)
plt.ylabel("x,y",fontsize=16)
plt.plot(resultarray[0,:],resultarray[1,:],'r',linewidth=2)
plt.plot(resultarray[0,:],resultarray[2,:],'b',linewidth=2)
plt.plot(maxt,maxr,'go',linewidth=2) # plot maxima of y(t)
#plt.savefig("RäuberBeute.pdf")

plt.figure(2)
plt.xlabel("x",fontsize=16)
plt.ylabel("y",fontsize=16)
plt.plot(resultarray[1,:],resultarray[2,:],'g',linewidth=2)
plt.plot(d/c,a/b, marker='o', markersize=10, markeredgecolor='m',mfc='None',markeredgewidth=2)
plt.plot(0,0,'ro', markersize=10, markeredgecolor='r',mfc='None',markeredgewidth=2)
plt.xlim([-100,10000.0])
plt.ylim([-10,200.0])

plt.figure(3)
plt.xlabel(r'$\mathrm{max}_{n}$',fontsize=18)
plt.ylabel(r'$\mathrm{max}_{n+1}$',fontsize=18)
plt.plot(np.array(maxr[0:len(maxr)-1]), np.array(maxr[1:len(maxr)]), 'bo')
#plt.xlim(3000.0,10000.00)
#plt.ylim(3000.0,10000.0)

plt.show()
